<?php

namespace Drupal\tinglejs\Form;

use Drupal\Core\Asset\LibraryDiscoveryInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\tinglejs\Exception\LibraryException;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure Tingle.js settings for this site.
 */
final class SettingsForm extends ConfigFormBase {

  /**
   * Config settings.
   *
   * @var string
   */
  const SETTINGS = 'tinglejs.settings';

  /**
   * The library.discovery service.
   *
   * @var \Drupal\Core\Asset\LibraryDiscoveryInterface
   */
  protected $libraryDiscovery;

  /**
   * Constructs a form object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config.factory service.
   * @param \Drupal\Core\Asset\LibraryDiscoveryInterface $library_discovery
   *   The library.discovery service.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    LibraryDiscoveryInterface $library_discovery
  ) {
    $this->libraryDiscovery = $library_discovery;
    parent::__construct($config_factory);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('library.discovery')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'tinglejs_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [static::SETTINGS];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $registered_libraries = $this->libraryDiscovery->getLibraryByName('tinglejs', TINGLEJS_LIBRARY_NAME);
    if (empty($registered_libraries)) {
      throw new LibraryException('Library "' . TINGLEJS_LIBRARY_NAME . '" could not be discovered.');
    }

    $config = $this->config(static::SETTINGS);
    $form['installation'] = [
      '#type' => 'select',
      '#title' => $this->t('Installation'),
      '#options' => [
        // Do not provide this library:
        'disabled' => $this->t('Disabled'),
        'local' => $this->t('Local library (manual download)'),
        // Local library downloaded via composer using bower-asset/tingle:
        // See https://www.drupal.org/docs/8/modules/lazy-load/how-to-use-composer-to-install-lazy-load-module-and-its-dependency
        'local_composer_npm' => $this->t('Local library via Composer (Asset Packagist: NPM)'),
        'local_composer_bower' => $this->t('Local library via Composer (Asset Packagist: Bower)'),
        // Load from CDN: CDNJS:
        'cdnjs' => $this->t('CDNJS'),
        // Load from CDN: jsDelivr:
        'jsdelivr' => $this->t('jsDelivr'),
      ],
      '#default_value' => $config->get('installation'),
      '#description' => $this->t("<strong>Source options:</strong><dl>
        <dt>Disabled:</dt><dd>Do not provide this library at all</dd>
        <dt>Local library (manual download):</dt><dd>Download the library manually and place it in <em>libraries/tinglejs</em> (resulting in <code>libraries/tinglejs/dist/tingle(.min).js</code>)</dd>
        <dt>Local via Composer (Asset Packagist: NPM):</dt><dd>Download libraries by <code>'composer require npm-asset/tingle.js:&lt;VERSION&gt;'</code> to libraries/tingle.js/dist/tingle(.min).js. See <a href='@composerAssetsInfoUrl'>this drupal.org Documentation</a> for details.</dd>
        <dt>Local via Composer (Asset Packagist: Bower):</dt><dd>Download libraries by <code>'composer require bower-asset/tingle:&lt;VERSION&gt;'</code> to libraries/tingle/dist/tingle(.min).js. See <a href='@composerAssetsInfoUrl'>this drupal.org Documentation</a> for details.</dd>
        <dt>CDNJS:</dt><dd>Loads the library from remote CDN <a href='https://cdnjs.com/'>https://cdnjs.com/</a></dd>
        <dt>jsDelivr:</dt><dd>Loads the library from remote CDM <a href='https://www.jsdelivr.com/'>https://www.jsdelivr.com/</a></dd>
        </dl>
        ", [
          '@composerAssetsInfoUrl' => 'https://www.drupal.org/docs/develop/using-composer/using-composer-to-install-drupal-and-manage-dependencies#third-party-libraries'
      ]),
    ];

    $form['development'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Development version'),
      '#default_value' => $config->get('development'),
      '#states' => [
        'invisible' => [
          'select[name="installation"]' => [
            'value' => 'disabled',
          ],
        ],
      ],
    ];

    $form['version'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Version'),
      '#size' => 9,
      '#required' => TRUE,
      '#default_value' => $config->get('version'),
      '#states' => [
        'invisible' => [
          'select[name="installation"]' => [
            ['value' => 'disabled'],
            ['value' => 'local_composer_npm'],
            ['value' => 'local_composer_bower'],
          ],
        ],
      ],
    ];

    $path = $registered_libraries['js'][0]['data'];
    if(!empty($path)){
      $options = [
        'absolute' => TRUE,
        'attributes' => ['target' => '_blank'],
      ];
      if (strpos($path, 'libraries') === 0) {
        $url = Url::fromUserInput('/' . $path, $options);
      }
      else {
        $url = Url::fromUri($path, $options);
      }

      $form['url'] = [
        '#type' => 'item',
        '#title' => $this->t('URL'),
        '#markup' => (new Link($url->toString(), $url))->toString(),
        '#states' => [
          'invisible' => [
            'select[name="installation"]' => ['value' => 'disabled'],
          ],
        ],
      ];
    }

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $version = $form_state->getValue('version');
    if (!preg_match('/^\d+\.\d+\.\d+$/', $version)) {
      $form_state->setErrorByName('version' . '][version', $this->t('Version format is not correct.'));
    }
    $form_state->setValue('version', $version);

    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->configFactory->getEditable(static::SETTINGS)
      ->set('installation', $form_state->getValue('installation'))
      ->set('development', $form_state->getValue('development'))
      ->set('version', $form_state->getValue('version'))
      ->save();

    $this->libraryDiscovery->clearCachedDefinitions();
    parent::submitForm($form, $form_state);
  }

}
